"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.level = void 0;

var _stream = require("stream");

var _types = require("postgres/lib/types");

const level = (sql, table) => {
  const created = sql`CREATE TABLE IF NOT EXISTS ${sql(table)} (
    key   TEXT  PRIMARY KEY,
    value JSONB
  )`;

  const put = async (key, token, fn) => {
    await created;

    try {
      await sql`
        INSERT INTO ${sql(table)} (key, value) VALUES (${key}, ${sql.json(token)})
        ON CONFLICT (key) DO UPDATE SET value = ${sql.json(token)}
      `;
      fn === null || fn === void 0 ? void 0 : fn(null, token);
    } catch (err) {
      fn === null || fn === void 0 ? void 0 : fn(err, null);
    }
  };

  const get = async (key, fn) => {
    await created;

    try {
      const [{
        value: value
      }] = await sql`SELECT value FROM ${sql(table)} WHERE key = ${key}`;
      fn === null || fn === void 0 ? void 0 : fn(null, value);
    } catch (err) {
      fn === null || fn === void 0 ? void 0 : fn(err, null);
    }
  };

  const del = async (key, fn) => {
    await created;

    try {
      await sql`DELETE FROM ${sql(table)} WHERE key = ${key}`;
      fn === null || fn === void 0 ? void 0 : fn(null);
    } catch (err) {
      fn === null || fn === void 0 ? void 0 : fn(err);
    }
  };

  const createReadStream = options => {
    const conditions = [];
    let order = 'ASC';
    const limit = options.limit;

    if (options.gt) {
      conditions.push({
        column: 'key',
        operator: '>',
        value: options.gt
      });
    }

    if (options.gte) {
      conditions.push({
        column: 'key',
        operator: '>=',
        value: options.gte
      });
    }

    if (options.lt) {
      conditions.push({
        column: 'key',
        operator: '<',
        value: options.lt
      });
    }

    if (options.lte) {
      conditions.push({
        column: 'key',
        operator: '<=',
        value: options.lte
      });
    }

    if (options.reverse) {
      order = 'DESC';
    }

    const params = [];
    const stream = new _stream.Readable();
    const query = created.then(() => sql.unsafe(`SELECT key, value FROM ${(0, _types.escape)(table)} WHERE TRUE AND ${conditions.map(condition => `${(0, _types.escape)(condition.column)} ${condition.operator} $${params.push(condition.value)}`).join(' AND ')} AND ORDER BY key ${order} ${limit ? `LIMIT ${limit}` : ''}`, params).stream(row => {
      stream.emit('data', {
        key: row.key,
        value: row.value
      });
    }));
    query.then(() => stream.emit('end')).catch(err => stream.emit('error', err));
    return stream;
  };

  return {
    put,
    get,
    del,
    createReadStream
  };
};

exports.level = level;